<?php

//require_once("ServerConfig.php");

// No time limit
set_time_limit(0);

// Config headers
define("CONFIG_HEADER_START", 				"NULL");
define("CONFIG_HEADER_ENVIRONMENT", 		"ENVIRONMENT");
define("CONFIG_HEADER_CONFIG", 				"CONFIG");
define("CONFIG_HEADER_ACCTYPES", 			"ACCTYPES");
define("CONFIG_HEADER_CLASSVARS", 			"CLASSVARS");
define("CONFIG_HEADER_MODULES", 			"MODULES");
define("CONFIG_HEADER_CLASSVAR_MAPPING",	"CLASSVAR_MAPPING");

// CLASSVAR_MAPPING
define("CLASSIFICATION_DATATYPE",	"CLASSIFICATION_DATATYPE");
define("CLASSIFICATION_ALLVALUES",	"CLASSIFICATION_ALLVALUES");
define("CLASSIFICATION_MAPSPEC",	"CLASSIFICATION_MAPSPEC");
define("CLASSIFICATION_MAPVALUE",	"CLASSIFICATION_MAPVALUE");


class ConfigReader {
	
	private $db;
	private $start;
	
	private $filename;
	private $data;
	
	private $mappingData;
	private $sql;
	
	private $addedValues;
	
	private $modules;
	
	private $accTypes;
	private $accTypesReverse;
	
	private $fh;
	
	private $cacheClassification;
	private $cacheClassificationValues;
	
	private $cache1;
	private $cache2;
	
	private $tempFilename;	
	private $databaseFilename;
	
	
	// ids
	private $classificationIds;
	private $classificationValueIds;
	

	private function wsql($sql) {
		fwrite($this->fh, $sql . "\n");
	}
	
	private function getClassificationId() {
		$this->classificationIds++;
		return $this->classificationIds;
	}	
	
	private function getClassificationValueId() {
		$this->classificationValueIds++;
		return $this->classificationValueIds;
	}
	
	function __construct($filename = null) {
		
		// Create new database filename
		$this->databaseFilename = date("U") . '.s3db';
		$this->tempFilename = ServerConfig::getDatabasePath() . $this->databaseFilename . '.temp';
				
		// so it starts at 0
		$this->classificationIds = -1;
		$this->classificationValueIds = -1;
		
		$this->fh = fopen($this->tempFilename, 'w');
		
		$this->start = date("U");
		$this->db = new PDO('sqlite:' . ServerConfig::getDatabasePath() . $this->databaseFilename);

		if (!$this->db) die("Sorry!");
		
		$this->accTypes = array();
		
		$this->filename = $filename;
		$this->wsql('
BEGIN;

CREATE TABLE "Module" (
	"id" INTEGER PRIMARY KEY NOT NULL,
	"value" VARCHAR(255) NULL
);
				
CREATE TABLE "DataType" (
	"id" INTEGER PRIMARY KEY NOT NULL,
	"moduleId" INTEGER  NULL,
	"value" VARCHAR(255) NULL
);
				
CREATE TABLE "Measurement" (
	"id" INTEGER PRIMARY KEY NOT NULL,
	"internalId" INTEGER  NULL,
	"datatypeId" INTEGER  NULL,
	"value" VARCHAR(255) NULL,
	"description" VARCHAR(500) NULL
);
				
CREATE TABLE "Classification" (
	"id" INTEGER PRIMARY KEY NOT NULL,
	"datatypeId" INTEGER  NULL,
	"internalId" INTEGER  NULL,
	"value" VARCHAR(255) NULL,
	"static" INTEGER NULL,
	"anonymous" INTEGER NULL,
	"virtualGui" INTEGER NULL
);
				
CREATE TABLE "ClassificationValue" (
	"id" INTEGER PRIMARY KEY NOT NULL,
	"classificationId" INTEGER  NULL,
	"value" VARCHAR(255) NULL,
	"anonymous" VARCHAR(255) NULL,
	"static" INTEGER NULL
);
				
CREATE TABLE "ValuesRelationship" (
	"leftValueId" INTEGER  NULL,
	"rightValueId" INTEGER  NULL
);');
	}
	
	function parse() {
		
		// Get data from file
		$this->data = @file($this->filename);
		if (!$this->data) {
			echo("Can't read data from source: " . $this->filename);
			return false;
		}

		// Init diffrent modes
		$mode = CONFIG_HEADER_START;
		$g = 0;
		
		// Go trhu each line from config
		foreach ($this->data as $lineNumber => $line) {
			
			// Remove white space on both sides
			$line = utf8_encode(trim($line));
			
			// If the line is empty, continue
			if ($line == "") continue;
			
			// Check for new config header
			switch ($line) {
				
				case CONFIG_HEADER_ENVIRONMENT:
					Logger::Debug("ConfigReader->parse", "FOUND NEW CONFIG HEADER (" . CONFIG_HEADER_ENVIRONMENT .")", $lineNumber);
					$mode = CONFIG_HEADER_ENVIRONMENT;
					continue 2;
					
				case CONFIG_HEADER_CONFIG:
					Logger::Debug("ConfigReader->parse", "FOUND NEW CONFIG HEADER (" . CONFIG_HEADER_CONFIG .")", $lineNumber);
					$mode = CONFIG_HEADER_CONFIG;
					continue 2;
					
				case CONFIG_HEADER_ACCTYPES:
					Logger::Debug("ConfigReader->parse", "FOUND NEW CONFIG HEADER (" . CONFIG_HEADER_ACCTYPES .")", $lineNumber);
					$mode = CONFIG_HEADER_ACCTYPES;
					continue 2;
					
				case CONFIG_HEADER_CLASSVARS:
					//continue 2;
					// This config part is "DEAD"! - Check with borgar if we could remove it complete?
					Logger::Debug("ConfigReader->parse", "FOUND NEW CONFIG HEADER (" . CONFIG_HEADER_CLASSVARS .")", $lineNumber);
					$mode = CONFIG_HEADER_CLASSVARS;
					continue 2;
					
				case CONFIG_HEADER_MODULES:
					Logger::Debug("ConfigReader->parse", "FOUND NEW CONFIG HEADER (" . CONFIG_HEADER_MODULES .")", $lineNumber);
					$mode = CONFIG_HEADER_MODULES;
					continue 2;
					
				case CONFIG_HEADER_CLASSVAR_MAPPING:
					Logger::Debug("ConfigReader->parse", "FOUND NEW CONFIG HEADER (" . CONFIG_HEADER_CLASSVAR_MAPPING .")", $lineNumber);
					$mode = CONFIG_HEADER_CLASSVAR_MAPPING;
					continue 2;
					
				default:
					break;
			}

			
			// Parsing data
			switch ($mode) {
				
				case CONFIG_HEADER_ENVIRONMENT:
					break;
					
				case CONFIG_HEADER_CONFIG:
					// We unset the license status value, since this needs to be set every time...
					ServerConfig::setLicenseStatus(false);
					$this->parseConfigHeader($line, $lineNumber);					
					break;
					
				case CONFIG_HEADER_ACCTYPES:
					$this->parseAcctypes($line, $lineNumber);
					break;
					
				case CONFIG_HEADER_CLASSVARS:
					break;
					
				case CONFIG_HEADER_MODULES:
					$this->parseModuleList($line, $lineNumber);
					break;
					
				case CONFIG_HEADER_CLASSVAR_MAPPING:
					$this->parseClassvarMapping($line, $lineNumber);
					break;
				
			}
			
			
		}
		

		// Add last line, and close the file
		$this->wsql('UPDATE Measurement SET internalId = internalId - 1;');
		$this->wsql('CREATE INDEX [C1]  ON [Classification]		( [id]  ASC );');
		$this->wsql('CREATE INDEX [C2]  ON [Classification]		( [datatypeId]  ASC );');
		$this->wsql('CREATE INDEX [CV1] ON [ClassificationValue] 	( [id]  ASC );');
		$this->wsql('CREATE INDEX [CV2] ON [ClassificationValue] 	( [value]  ASC );');
		$this->wsql('CREATE INDEX [CV3] ON [ClassificationValue] 	( [classificationId]  ASC );');
		$this->wsql('CREATE INDEX [VR1] ON [ValuesRelationship] 	( [leftValueId]  ASC );');
		$this->wsql('CREATE INDEX [VR2] ON [ValuesRelationship] 	( [rightValueId]  ASC );');		
		$this->wsql('COMMIT;');
		fclose($this->fh);
		
		// Execute the SQL
		if ($this->db->exec(file_get_contents($this->tempFilename))) {
			// Remove temp file
			unlink($this->tempFilename);
			
			// update the ini file
			ServerConfig::setDatabase($this->databaseFilename);
			
			// Add Virtual GUI Classifications
			GuiMappingManager::run();
		}

	}
	
	
	// Parsing the CONFIG section from the dump
	private function parseConfigHeader($line, $lineNumber) {
		list($key, $value) = explode(":", $line);
		
		switch (strtoupper($key)) {
			
			case "LICENSE_STATUS_ENABLED":
				ServerConfig::setLicenseStatus(true);
				break;			
		}
		
	}
	
	private function parseModuleList($line, $lineNumber) {
		list($name, $accTypeIDs) = explode(":", $line);
		$accTypeIDs = explode(",", $accTypeIDs);	
		
		$this->modules[$name] = $accTypeIDs;
		$this->wsql('INSERT INTO Module VALUES(' . (count($this->modules)-1) . ', "' . escape_string($name) . '");');
			
		// Update DataTypes
		foreach ($accTypeIDs as $accTypeID) {
			$this->wsql('UPDATE DataType SET moduleId = ' . (count($this->modules)-1) . ' WHERE id = ' . $accTypeID . ';');
		}
	}
	
	
	private function findAcctype($name) {
		if (empty($this->accTypesReverse)) {
			$this->accTypesReverse = array();
			foreach ($this->accTypes as $id => $acctype) {
				$this->accTypesReverse[$acctype] = $id;
			}
		}

		if (isset($this->accTypesReverse[$name])) {
			return $this->accTypesReverse[$name];
		}

			$this->accTypesReverse = array();
			foreach ($this->accTypes as $id => $acctype) {
				$this->accTypesReverse[$acctype] = $id;
			}

		if (isset($this->accTypesReverse[$name])) {
			return $this->accTypesReverse[$name];
		}
			
		die("No ACCTYPE!");
	}
	
	private function parseAcctypes($line, $lineNumber) {
		
		list($id, $name, $components) = explode(":", $line);
		list ($measurements, $classifications) = explode(", #,", $components);
		
		$measurements = explode(",", $measurements);
		$classifications = explode(",", $classifications);
		
		// Add to internal list
		$this->accTypes[] = escape_string($name);
		
		// Stop parsing if name is "Reserved"
		if (substr_count(strtoupper($name), "RESERVED") > 0) {
			Logger::Info("ConfigReader->parseAcctypes", "NOT INCLUDING ACCTYPE (" . $name ."). THE NAME 'RESERVED' MEANS ITS ONLY RESERVED, NOT IN USE", $lineNumber);
			return;
		}
		
		// Stop parsing if name is "AvailReporter"
		if (substr_count(strtoupper($name), "AVAILREPORTER") > 0) {
			Logger::Info("ConfigReader->parseAcctypes", "NOT INCLUDING ACCTYPE (" . $name ."). THIS ACCTYPE IS DEPRECATED", $lineNumber);
			return;
		}
		
		// Check if there is a name with the acctype
		if (trim($name) == "") {
			Logger::Error("ConfigReader->parseAcctypes", "SKIPPING ACCTYPE - NO NAME", $lineNumber);
			return;		
		}
		
		// Loggin
		Logger::Debug("ConfigReader->parseAcctypes", "FOUND NEW ACCTYPE (" . $name .")", $lineNumber);
		
		
		// Insert data type
		$this->wsql('INSERT INTO DataType VALUES(' . (count($this->accTypes)-1) . ', 0, "' . escape_string($name) . '");' . "\n");
		
		// Insert Classifications
		$internalId = 0;
		foreach ($classifications as $classification) {
			$classification = trim($classification);
			if ($classification != "") {

				// Internal id is always based on the order
				$internalId++;
				
				// cache
				$uid = $this->getClassificationId();
				$this->setClassificationCache($this->findAcctype($name), $classification, $uid);
				
				// Check if the classification supports anonymous
				if (substr($classification, 0, 2) == "£") {
					$classification = substr($classification, 2);					
					$this->wsql('INSERT INTO Classification VALUES (' . $uid . ', ' . $this->findAcctype($name) . ', '.$internalId.', "' . $classification . '", 0, 1, 0);');
				} else {
					$this->wsql('INSERT INTO Classification VALUES (' . $uid . ', ' . $this->findAcctype($name) . ', '.$internalId.', "' . $classification . '", 0, 0, 0);');
				}
				
			}
		}
		
		// Insert Measurements
		$internalId = 0;
		foreach ($measurements as $measurement) {
			$pos = strpos($measurement, "(");
			if ($pos === false) {
				$measurement_name = trim($measurement);
				$measurement_desc = "";
			} else {
				$measurement_name = trim(substr($measurement, 0, $pos));
				$measurement_desc = trim(substr($measurement, $pos+1, strrpos($measurement, ")")-$pos-1));
			}
			
			// Internal id is always based on the order
			$internalId++;
			
			// Adding..
			$this->wsql('INSERT INTO Measurement VALUES(null, ' . $internalId . ', ' . $this->findAcctype($name) . ', "' . escape_string($measurement_name) . '", "' . escape_string($measurement_desc) . '");');			
		}
		
		return;
	}
	
	
	private function parseClassvarMapping($line, $lineNumber) {
		
		list ($subMode, $data) = explode(":", $line, 2);
		
		switch ($subMode) {
			
			case CLASSIFICATION_DATATYPE:
				$data = trim($data);
				Logger::Debug("ConfigReader->parseClassvarMapping", "FOUND NEW CALSSIFICATION DATATYPE (" . $data .")", $lineNumber);
				$this->mappingData = array(
					CLASSIFICATION_DATATYPE => $data,
					CLASSIFICATION_MAPVALUE => array(),
					CLASSIFICATION_MAPVALUE . "_CLONE" => array(),
					"CLASSIFICATION_MAPSPEC2" => array()
				);			
				
				$this->cache1 = array();	
				$this->cache2 = array();	
				
				// This list make sure that we don't add a Value twice
				$this->addedValues = array();				
				break;
				
			case CLASSIFICATION_ALLVALUES:
				if (!isset($this->mappingData) || !is_array($this->mappingData) || !isset($this->mappingData[CLASSIFICATION_DATATYPE])) {
					Logger::Error("ConfigReader->parseClassvarMapping", "CAN’T ADD 'ALL VALUES' TO A DATA TYPE THAT DOESN’T EXISTS!", $lineNumber);
					break;
				}
				
				// Split the data to two valiables
				list ($source, $values) = explode(":", $data, 2);
				$source = trim($source);
				$values = explode(":", $values);
				
				if ($source == "" || !is_array($values) || count($values) == 0) {
					Logger::Error("ConfigReader->parseClassvarMapping", "CAN’T ADD 'ALL VALUES' TO DATATYPE (" . $this->mappingData[CLASSIFICATION_DATATYPE] . "), SOURCE IS BLANK OR THERE IS NONE VALID VALUES", $lineNumber);
					break;					
				}
				
				Logger::Debug("ConfigReader->parseClassvarMapping", "ADDING NEW 'ALL VALUES' TO DATATYPE (" . $this->mappingData[CLASSIFICATION_DATATYPE] . ")", $lineNumber);

				// Changing Classification to be static
				$this->wsql('UPDATE Classification SET static=1 WHERE datatypeId = ' . $this->findAcctype($this->mappingData[CLASSIFICATION_DATATYPE]) . ' and value = "' . $source . '";');

				// classification id from cache
				$classificationId = $this->findClassification($this->mappingData[CLASSIFICATION_DATATYPE], $source);
				
				
				// Add values
				foreach ($values as $value) {
					if (trim($value) == "") continue;
					
					if (!isset($this->addedValues[$classificationId . $value])) {
						$uid = $this->getClassificationValueId();
						
						// Check if data contains anonymous values
						if (strpos($value, "£") != false) {
							$normal = substr($value, 0, strpos($value, "£"));
							$anonymous = substr($value, strpos($value, "£")+2);
							$this->wsql('INSERT INTO ClassificationValue VALUES(' . $uid . ', ' . $classificationId . ', "' . escape_string($normal) . '", "' . escape_string($anonymous) . '", 1);');
							$this->setClassificationValueCache($classificationId, $normal, $uid);
						} else {
							$this->wsql('INSERT INTO ClassificationValue VALUES(' . $uid . ', ' . $classificationId . ', "' . escape_string($value) . '", null, 1);');
							$this->setClassificationValueCache($classificationId, $value, $uid);
						}
						
						// Add to list
						$this->addedValues[$classificationId . $value] = 1;
					}
				}
				break;
				
			
				
			case CLASSIFICATION_MAPSPEC:
				if (!isset($this->mappingData) || !is_array($this->mappingData) || !isset($this->mappingData[CLASSIFICATION_DATATYPE])) {
					Logger::Error("ConfigReader->parseClassvarMapping", "FOUND 'MAP SPEC' BUT CAN’T HANDLE  TO A DATA TYPE THAT DOESN’T EXISTS!", $lineNumber);
					break;
				}
				
				// Split the data to two map's
				list ($mapA, $mapB, $NA) = explode(":", $data, 3);
				$mapA = trim($mapA);
				$mapB = trim($mapB);
				
				if ($mapA == "" || $mapB == "") {
					Logger::Error("ConfigReader->parseClassvarMapping", "FOUND 'MAP SPEC' BUT CAN’T CREATING MAPPINGS WHERE ONE/OR MORE OF THE VALUES ARE EMPTY", $lineNumber);
					break;					
				}
				
				Logger::Debug("ConfigReader->parseClassvarMapping", "FOUND NEW 'MAP SPEC' (" . $mapA . " <=> " . $mapB . ") TO DATATYPE (" . $this->mappingData[CLASSIFICATION_DATATYPE] . ")", $lineNumber);
				$this->mappingData[CLASSIFICATION_MAPSPEC] = array($mapA, $mapB);
				
				// Add Classification s if the don’t exists
				if (!in_array($mapA, $this->mappingData[CLASSIFICATION_MAPSPEC . "2"])) {
					$this->mappingData[CLASSIFICATION_MAPSPEC . "2"][] = $mapA;
				}
				
				if (!in_array($mapB, $this->mappingData[CLASSIFICATION_MAPSPEC . "2"])) {
					$this->mappingData[CLASSIFICATION_MAPSPEC . "2"][] = $mapB;
				}
				
				
				
				break;
				
			case CLASSIFICATION_MAPVALUE:
				if (!isset($this->mappingData) || !is_array($this->mappingData) || !isset($this->mappingData[CLASSIFICATION_DATATYPE])) {
					Logger::Error("ConfigReader->parseClassvarMapping", "FOUND 'MAP VALUE' BUT CAN’T HANDLE TO A DATA TYPE THAT DOESN’T EXISTS!", $lineNumber);
					break;
				}
				
				if (!isset($this->mappingData[CLASSIFICATION_MAPSPEC]) || count($this->mappingData[CLASSIFICATION_MAPSPEC]) != 2) {
					Logger::Error("ConfigReader->parseClassvarMapping", "FOUND 'MAP VALUE' BUT CAN’T HANDLE IT, SINCE THERE IS NONE 'MAP SPEC' FOR THE VALUES", $lineNumber);
					break;
				}
				
				Logger::Debug("ConfigReader->parseClassvarMapping", "FOUND NEW 'MAP VALUE' TO DATATYPE '" . $this->mappingData[CLASSIFICATION_DATATYPE] . "' WITH THE 'MAP SPEC' '" . $this->mappingData[CLASSIFICATION_MAPSPEC][0] . " <=> " . $this->mappingData[CLASSIFICATION_MAPSPEC][1]  . "'", $lineNumber);
				
				// Get the source and all the values seperated
				list ($source, $values) = explode(":", $data, 2);
				$source = trim($source);
				$values = explode(":", $values);
				
				if ($source == "" || $value == ' ') {
				 	Logger::Info("ConfigReader->parseClassvarMapping", "VALUE IN MAPPING IS EMPTY, REPLACING IT WITH <EMPTY>", $lineNumber);
					$source = "<empty>";
					//Logger::Info("ConfigReader->parseClassvarMapping", "SOURCE VALUE IN MAPPING IS EMPTY, SKIPPING IT", $lineNumber);
					//break;
				}

				// classification id from cache
				$classificationId_p = $this->findClassification($this->mappingData[CLASSIFICATION_DATATYPE], $this->mappingData[CLASSIFICATION_MAPSPEC][0]);
				$classificationId_s = $this->findClassification($this->mappingData[CLASSIFICATION_DATATYPE], $this->mappingData[CLASSIFICATION_MAPSPEC][1]);

				// Check if the source is added before, if not, add it
				if (!isset($this->addedValues[$classificationId_p . $source])) {
					$uid = $this->getClassificationValueId();
					
					// Check if data contains anonymous values
					if (strpos($source, "£") != false) {
						$normal = substr($source, 0, strpos($source, "£"));
						$anonymous = substr($source, strpos($source, "£")+2);
						$this->wsql('INSERT INTO ClassificationValue VALUES(' . $uid . ', ' . $classificationId_p . ', "' . escape_string($normal) . '", "' . escape_string($anonymous) . '", 1);');
						$this->setClassificationValueCache($classificationId_p, $normal, $uid);
					} else {
						$this->wsql('INSERT INTO ClassificationValue VALUES(' . $uid . ', ' . $classificationId_p . ', "' . escape_string($source) . '", null, 1);');
						$this->setClassificationValueCache($classificationId_p, $source, $uid);
					}

					// Add to list
					$this->addedValues[$classificationId_p . $source] = 1;
					
					$this->mappingData[CLASSIFICATION_MAPVALUE][] = $this->mappingData[CLASSIFICATION_MAPSPEC][0] . $source;
					$this->mappingData[CLASSIFICATION_MAPVALUE. "_CLONE"][$this->mappingData[CLASSIFICATION_MAPSPEC][0] . $source] = 1;
				}
				
				
				// For each value add it
				$tt = count($values);
				foreach ($values as $valueLine => $value) {
					$value = trim($value);

					if($value == "" || $value == ' '){
						Logger::Info("ConfigReader->parseClassvarMapping", "VALUE IN MAPPING IS EMPTY, REPLACING IT WITH <EMPTY>", $lineNumber);
						$value = "<empty>";
						//Logger::Info("ConfigReader->parseClassvarMapping", "VALUE IN MAPPING IS EMPTY, SKIPPING IT", $lineNumber);
						//continue;
					}					
					
					// Check if the value is added before, if not, add it
					if (!isset($this->addedValues[$classificationId_s . $value])) {
					
						$uid = $this->getClassificationValueId();
					
						// Check if data contains anonymous values
						if (strpos($value, "£") != false) {
							$normal = substr($value, 0, strpos($value, "£"));
							$anonymous = substr($value, strpos($value, "£")+2);
							$this->wsql('INSERT INTO ClassificationValue VALUES(' . $uid . ', ' . $classificationId_s . ', "' . escape_string($normal) . '", "' . escape_string($anonymous) . '", 0);');
							$this->setClassificationValueCache($classificationId_s, $normal, $uid);
						} else {
							$this->wsql('INSERT INTO ClassificationValue VALUES(' . $uid . ', ' . $classificationId_s . ', "' . escape_string($value) . '", null, 0);');
							$this->setClassificationValueCache($classificationId_s, $value, $uid);
						}
				
						// Add to list
						$this->addedValues[$classificationId_s . $value] = 1;
						Logger::Debug("ConfigReader->parseClassvarMapping", "Adding  '" . $value . "' in ".  $this->mappingData[CLASSIFICATION_MAPSPEC][1]);
								
						$this->mappingData[CLASSIFICATION_MAPVALUE][] = $this->mappingData[CLASSIFICATION_MAPSPEC][1] . $value;
						$this->mappingData[CLASSIFICATION_MAPVALUE. "_CLONE"][$this->mappingData[CLASSIFICATION_MAPSPEC][1] . $value] = 1;
					}
					
					// Add mapping
					if (strpos($source, "£") != false) {
						$source = substr($source, 0, strpos($source, "£"));
					}
					if (strpos($value, "£") != false) {
						$value = substr($value, 0, strpos($value, "£"));
					}
					
					$this->wsql('INSERT INTO ValuesRelationship VALUES(' . $this->findClassificationValue($classificationId_p, $source) . ', ' . $this->findClassificationValue($classificationId_s, $value) . ');');
					}
					
				break;
					
		}
		
	}
	
	private function setClassificationCache($dataTypeId, $classification, $id) {
		if (!is_array($this->cacheClassification)) $this->cacheClassification = array();
		if (!isset($this->cacheClassification[$dataTypeId])) $this->cacheClassification[$dataTypeId] = array();
		$this->cacheClassification[$dataTypeId][str_replace("£", "", $classification)] = $id;
	}
	
	private function findClassification($dataType, $name) {
		
		$dataTypeId = $this->findAcctype($dataType);
		if (isset($this->cacheClassification[$dataTypeId][$name])) {
			return $this->cacheClassification[$dataTypeId][$name];
		}
		
		// If its not in, we need to create one..
		$newID = count($this->cacheClassification[$dataTypeId]);
		$this->cacheClassification[$dataTypeId][str_replace("£", "", $name)] = $newID;
		return $newID;
	}	
	
	private function setClassificationValueCache($classificationId, $value, $id) {
		if (!is_array($this->cacheClassificationValues)) $this->cacheClassificationValues = array();
		if (!isset($this->cacheClassificationValues[$classificationId])) $this->cacheClassificationValues[$classificationId] = array();
		$this->cacheClassificationValues[$classificationId][$value] = $id;
	}
	
	private function findClassificationValue($classificationId, $name) {
		if (isset($this->cacheClassificationValues[$classificationId][$name])) {
			return $this->cacheClassificationValues[$classificationId][$name];
		}
	}	

}

?>